@@ -29,6 +29,8 @@
 #include "pango-font.h"
 #include "pango-utils.h"
 
+#include <glib/gstdio.h>
+
 #ifdef HAVE_FRIBIDI
 #include <fribidi/fribidi.h>
 #endif
@@ -56,7 +58,7 @@ struct PangoAlias
   gboolean visible; /* Do we want/need this? */
 };
 
-GHashTable *pango_aliases_ht = NULL;
+static GHashTable *pango_aliases_ht = NULL;
 
 /**
  * pango_trim_string:
@@ -85,7 +87,7 @@ pango_trim_string (const char *str)
 
 /**
  * pango_split_file_list:
- * @str: a comma separated list of filenames
+ * @str: a G_SEARCHPATH_SEPARATOR separated list of filenames
  * 
  * Splits a G_SEARCHPATH_SEPARATOR-separated list of files, stripping
  * white space and subsituting ~/ with $HOME/.
@@ -131,6 +133,11 @@ pango_split_file_list (const char *str)
 	  g_free (file);
 	  file = tmp;
 	}
+      else if (file[0] == '~' && file[1] == '\0')
+	{
+	  g_free (file);
+	  file = g_strdup (g_get_home_dir());
+	}
 #endif
       g_free (files[i]);
       files[i] = file;
@@ -459,7 +466,7 @@ read_config_file (const char *filename, gboolean enoent_error)
   char *section = NULL;
   int line = 0;
 
-  file = fopen (filename, "r");
+  file = g_fopen (filename, "r");
   if (!file)
     {
       if (errno != ENOENT || enoent_error)
@@ -728,6 +735,7 @@ pango_parse_style (const char *str,
 	}
       break;
     case 'i':
+    case 'I':
       if (g_ascii_strcasecmp (str, "italic") == 0)
 	{
 	  *style = PANGO_STYLE_ITALIC;
@@ -735,6 +743,7 @@ pango_parse_style (const char *str,
 	}
       break;
     case 'o':
+    case 'O':
       if (g_ascii_strcasecmp (str, "oblique") == 0)
 	{
 	  *style = PANGO_STYLE_OBLIQUE;
@@ -1485,7 +1494,7 @@ read_alias_file (const char *filename)
   int n_new;
   int i;
 
-  file = fopen (filename, "r");
+  file = g_fopen (filename, "r");
   if (!file)
     return;
 
@@ -1705,3 +1714,37 @@ pango_find_base_dir (const gchar *text,
 
   return dir;
 }
+
+/**
+ * pango_is_zero_width:
+ * @ch: a unicode character
+ *
+ * Checks @ch to see if it is a zero-width character that should not be
+ * normally rendered on the screen.
+ *
+ * Return value: %TRUE if @ch is a zero-width character, %FALSE otherwise
+ */
+gboolean
+pango_is_zero_width (gunichar ch)
+{
+/* Zero Width characters:
+ *
+ *  200B  ZERO WIDTH SPACE
+ *  200C  ZERO WIDTH NON-JOINER
+ *  200D  ZERO WIDTH JOINER
+ *  200E  LEFT-TO-RIGHT MARK
+ *  200F  RIGHT-TO-LEFT MARK
+ *  2028  LINE SEPARATOR
+ *  202A  LEFT-TO-RIGHT EMBEDDING
+ *  202B  RIGHT-TO-LEFT EMBEDDING
+ *  202C  POP DIRECTIONAL FORMATTING
+ *  202D  LEFT-TO-RIGHT OVERRIDE
+ *  202E  RIGHT-TO-LEFT OVERRIDE
+ *  FEFF  ZERO WIDTH NO-BREAK SPACE
+ */
+  return (ch & ~(gunichar)0x003F == 0x2000 && (
+		(ch >= 0x200B && ch <= 0x200F) ||
+		ch == 0x2028 ||
+		(ch >= 0x202A && ch <= 0x202E)
+	 )) || ch == 0xFEFF;
+}
